# This file was created automatically by SWIG.
import progressbarc
from oof2.SWIG.common.threadstate import ThreadStatePtr
class ProgressPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def reset(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_reset,(self,) + _args, _kwargs)
        return val
    def stop_it(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_stop_it,(self,) + _args, _kwargs)
        return val
    def query_stop(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_query_stop,(self,) + _args, _kwargs)
        return val
    def set_success(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_set_success,(self,) + _args, _kwargs)
        return val
    def set_failure(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_set_failure,(self,) + _args, _kwargs)
        return val
    def get_success(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_get_success,(self,) + _args, _kwargs)
        return val
    def get_threadstate(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_get_threadstate,(self,) + _args, _kwargs)
        if val: val = ThreadStatePtr(val) 
        return val
    def get_progress(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_get_progress,(self,) + _args, _kwargs)
        return val
    def set_progress(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_set_progress,(self,) + _args, _kwargs)
        return val
    def set_message(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_set_message,(self,) + _args, _kwargs)
        return val
    def get_message(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_get_message,(self,) + _args, _kwargs)
        return val
    def cleanup(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_cleanup,(self,) + _args, _kwargs)
        return val
    def make_defunct(self, *_args, **_kwargs):
        val = apply(progressbarc.Progress_make_defunct,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C Progress instance at %s>" % (self.this,)
class Progress(ProgressPtr):
    def __init__(self,this):
        self.this = this






#-------------- FUNCTION WRAPPERS ------------------

def getProgress(*_args, **_kwargs):
    val = apply(progressbarc.getProgress,_args,_kwargs)
    if val: val = ProgressPtr(val)
    return val



#-------------- VARIABLE WRAPPERS ------------------

INCOMPLETE = progressbarc.INCOMPLETE
COMPLETE = progressbarc.COMPLETE
FAILED = progressbarc.FAILED


#-------------- USER INCLUDE -----------------------

# -*- python -*-
# $RCSfile: progressbar.spy,v $
# $Revision: 1.10 $
# $Author: langer $
# $Date: 2007-02-07 19:08:48 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common import debug
from oof2.common import utils
from oof2.common.IO import threadmanager


from oof2.SWIG.common import switchboard


_progressbar_options = [
    "continuous",
    "active",
 ##   "discrete"
    ]

## A ProgressBar is an object created by the worker in threaded mode
## and placed in the threadManager. These objects constitute the base
## for instantiating a representation (text or graphic), when an
## activityViewer window opens. See (***)

## When the activityViewer window is closed, the ProgressBar
## representations are destroyed, but the ProgressBar objects (the
## "core" of the progress bars) remain. 

## If an activityViewer window is already open and a new ProgressBar
## is created, a switchboard call message is issued and the activityViewer
## is automatically updated.

## The progress bar representation hierarchy is the following:
##
## A ProgressBarRepresentation is the base of all the graphic, text,
## acoustic, tickle, or otherwise UI that the user will experience (maybe
## one day in stereo).
##
## (***) The correct representation is instantiated by calling the function
## set_ProgressBarRepresentation, which is over-written in graphic mode.
##
## The TextProgressBar is a subclass of the ProgressBarRepresentation and
## has all the functionality to interact with the user in text mode.
##
## The GUIProgressBar is s subclass of the ProgressBarRepresentation and
## has all the functionality to interact with the user in graphics mode.


## IMPORTANT NOTE on the ProgressObject class:

## A ProgressObject is NOT derived from the Progress class
## because when one is instantiated within a thread it checks
## that the C++ counter-part is (or not is) already in the static
## list of progress objects. The ProgressObject object possesses a
## reference to the correct Progress object, which is shared between
## the C++ and the Python levels.
## The functionality of the ProgressObject is inherited through the
## use of a __getattr__ function.

class ProgressObject :
    def __init__(self, type = "continuous", name = None):
        self.progressbar = getProgress()
        if type in _progressbar_options: ## check validity
            self.type = type ## assign progressbar type
        else:
            raise AttributeError("progress bar option not valid")
        self.Worker = None
        self.name = name
        self.defunct = 0                # used by threadmanager
        
        ## After progress object is created, pass it to threadManager
        threadmanager.threadManager.newProgressBar(self)
        
    def __getattr__(self, attrname):
        ## Please see comment above
        return getattr(self.progressbar, attrname)
    
    def get_name(self):
        return self.name
    
    def set_name(self, name):
        self.name = name
        
    def registerWorker(self, wrkr):
        self.Worker = wrkr
        
    def unregisterWorker(self):
        self.Worker = None

    def get_type(self):
        return self.type
        
    def get_Progress(self):
        return self.progressbar

    
#########################################################################
#########################################################################
## The following is the basic class for creating a progress bar
## representation. Each representation has different API for drawing
## the progress bar.
#########################################################################
#########################################################################
    
class ProgressBarRepresentation:
    def __init__(self, progressBar):
        # progressBar is actually a ProgressObject instance
        self.progressbar = progressBar
        self.type = self.progressbar.get_type()
        self.name = self.progressbar.get_name()
        self.talk = 1
    def get_name(self):
        return self.progressbar.get_name()
    def set_progress(self, improvement):
        return self.progressbar.set_progress(improvement)
        
#########################################################################
#########################################################################
## The following describes the progress bar representation for a text
## progress bar. In this case, the progress bar object returnd a string
## whose length and contents depends on the current state of the thread.
#########################################################################
#########################################################################
        
class TextProgressBar(ProgressBarRepresentation):
    def __init__(self, progressBar):
        ProgressBarRepresentation.__init__(self, progressBar)
        self.window_width = 80
        self.bar_repr = None
        self.total_bar = " "
        self.window_width = utils.screenwidth()
        ## shuttle mode progress bar
        self.direction = 1
        self.counter = 0

    def bar_size(self):
        possible_sz =  self.window_width-len(self.progressbar.get_message())-5
        if possible_sz > 0:
            return possible_sz
        return 5
    
    ## functions to draw different types of progress bars
    pbchars = r"-------\|/"
    def progress_active(self):
        bar_length = self.bar_size()
        self.counter  += self.direction
        if self.counter+1 > bar_length:
            self.counter = 2*(bar_length-1) - self.counter
            self.direction = -1
        elif self.counter < 0:
            self.counter = -self.counter
            self.direction = 1
        pbchr = self.pbchars[self.counter%len(self.pbchars)]
        actual_bar = " "*self.counter + pbchr + " "*(bar_length-self.counter-1)
        return actual_bar

    def progress_incremental(self, progress):
        actual_bar = ""
        bar_length = self.bar_size()
        nstars = int(bar_length*progress)
        return "*"*nstars + " "*(bar_length - nstars)

    def buildprogressbar(self):
        ## Updates text progress bar.
        ## For activity mode, the passed value 
        ## is irrelevant. Valid values range 
        ## between zero and one.
        string_fraction = ""
        if self.type == "active":
            string_fraction = self.progress_active()
        elif self.type == "continuous": # or "discrete":
            string_fraction = self.progress_incremental(
                self.progressbar.get_progress())
        self.total_bar = (self.progressbar.get_message() + " |" +
                          string_fraction + "| " + "\r")
        return self.total_bar

#####################################################
    
class NullProgressBar(ProgressBarRepresentation):
    def buildprogressbar(self):
        pass

#####################################################    

_suppressProgressBars = False

def set_ProgressBarRepresentation(pbar): ## returns a text progress bar
    ## argument is a ProgressObject.. This function is redefined if the
    ## GUI is loaded.
    if _suppressProgressBars:
        return NullProgressBar(pbar)
    return TextProgressBar(pbar)
    


def suppressProgressBars():
    global _suppressProgressBars
    _suppressProgressBars = True
