# -*- python -*-
# $RCSfile: widgetlogger.py,v $
# $Revision: 1.5 $
# $Author: langer $
# $Date: 2007-08-14 18:45:33 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import loggers
import gtk
import logutils

import string

class WidgetLogger(loggers.GtkLogger):
    classes = (gtk.Widget,)
    events = {"button-press-event"  : "BUTTON_PRESS",
              "button-release-event": "BUTTON_RELEASE",
              "motion-notify-event" : "MOTION_NOTIFY"}
    buttonup = True
    def location(self, widget, *args):
        name = logutils.getWidgetName(widget)
        if not name:
            raise logutils.GtkLoggerException("Unnamed widget")
        path = self._parentWidgetPath(widget) + [name]
        if path[0] not in logutils.getTopWidgetNames():
            raise logutils.GtkLoggerException(string.join(path, ':') + 
                                     " is not contained in a top-level widget")
        return "findWidget('%s')" % string.join(path, ':')
    def _parentWidgetPath(self, widget):
        parent = widget.get_parent()
        while parent is not None and logutils.getWidgetName(parent) is None:
            parent = parent.get_parent()
        if parent is None:
            return []
        return self._parentWidgetPath(parent) + [logutils.getWidgetName(parent)]
    def record(self, object, signal, *args):
        eventname = self.events.get(signal, None)
        if eventname is not None:
            evnt = args[0]
            if eventname == 'BUTTON_PRESS':
                WidgetLogger.buttonup = False
            elif eventname == 'BUTTON_RELEASE':
                WidgetLogger.buttonup = True
            elif eventname == 'MOTION_NOTIFY':
                if logutils.suppress_motion_events() and WidgetLogger.buttonup:
                    return self.ignore
            wvar = loggers.localvar('widget')
            return [
                "%s = %s" % (wvar, self.location(object, *args)),
                "%s.event(event(gtk.gdk.%s,x=%20.13e,y=%20.13e,state=%d,window=%s.window))" % (wvar, eventname, evnt.x, evnt.y, evnt.state, wvar)
                ]

        if signal == 'focus-in-event':
            wvar = loggers.localvar('widget')
            return [
       "%s=%s" % (wvar, self.location(object, *args)),
       "%(widget)s.event(event(gtk.gdk.FOCUS_CHANGE, in_=1, window=%(widget)s.window))" % dict(widget=wvar)
                ]
        if signal == 'focus-out-event':
            wvar = loggers.localvar('widget')
            return [
       "%s=%s" % (wvar,self.location(object, *args)),
       "%(widget)s.event(event(gtk.gdk.FOCUS_CHANGE, in_=0, window=%(widget)s.window))" % dict(widget=wvar)
                ]
            
        if signal == 'size-allocate':
            alloc = object.get_allocation()
            parent = object.get_parent()
            return ["%s.size_allocate(gtk.gdk.Rectangle(%d, %d, %d, %d))" \
                   % (self.location(object, *args),
                      alloc.x, alloc.y, alloc.width, alloc.height)]
        return super(WidgetLogger, self).record(object, signal, *args)
