# -*- python -*-
# $RCSfile: director.py,v $
# $Revision: 1.11 $
# $Author: langer $
# $Date: 2005-05-13 18:43:45 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# The "director" enum class.  Used by the boundary constructor,
# but in common because it just might be useful elsewhere also.
from oof2.common import enum
from oof2.common import utils

class Director(enum.EnumClass('Clockwise','Counterclockwise',
                              'Left to right', 'Right to left',
                              'Top to bottom', 'Bottom to top')):
    tip = "Directions for arranging objects."
    discussion = """<para>
    <classname>Director</classname> objects are used to specify the
    direction of things like &skel; <link
    linkend='Section:Concepts:Skeleton:Boundary:Edge'>edge
    boundaries</link>.
    </para>"""


utils.OOFdefine('Director', Director)

# Toplogy hints, used by the DirectorWidget.
loopables = ['Clockwise', 'Counterclockwise']
unloopables = ['Left to right', 'Right to left',
               'Top to bottom', 'Bottom to top']

# Trivial subclass allows directors to have a custom widget.
class DirectorParameter(enum.EnumParameter):
    def __init__(self, name, value=None, default=None, tip=None):
        enum.EnumParameter.__init__(self, name, Director, value, default, tip)
