# -*- python -*-
# $RCSfile: meshbdyanalysis.py,v $
# $Revision: 1.3 $
# $Author: langer $
# $Date: 2006-07-27 14:31:36 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.engine import bdyanalysis
from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.engine.IO import meshparameters

class MeshBdyAnalyzer(registeredclass.RegisteredClass):
    registry = []
    tip = "Post-solution computations on Mesh boundaries."
    discussion = """<para>
    <classname>MeshBdyAnalyzer</classname> objects are used as the
    <varname>analyzer</varname> argument to the <xref
    linkend="MenuItem:OOF.Mesh.Boundary_Analysis"/> command.  They
    represent different types of calculations that can be done on the
    boundary of a Mesh after it's been solved.
    </para>"""

class IntegrateBdyFlux(MeshBdyAnalyzer):
    def __init__(self, flux):
        self.flux = flux
    def __call__(self, femesh, edgeset):
        return bdyanalysis.integrateFlux(femesh, self.flux, edgeset)

registeredclass.Registration(
    "Integrate_Flux",
    MeshBdyAnalyzer,
    IntegrateBdyFlux,
    ordering=2,
    params=[meshparameters.FluxParameter('flux', tip=parameter.emptyTipString)],
    tip="Integrate a Flux along a boundary.",
    discussion="""<para>
    Integrate the normal component of the given &flux; along a boundary.
    </para>""")


class AverageField(MeshBdyAnalyzer):
    def __init__(self, field):
        self.field = field
    def __call__(self, femesh, edgeset):
        return bdyanalysis.averageField(femesh, self.field, edgeset)


registeredclass.Registration(
    "Average Field",
    MeshBdyAnalyzer,
    AverageField,
    ordering=1,
    params=[meshparameters.FieldParameter('field',
                                          tip=parameter.emptyTipString)],
    tip="Average a Field over a boundary.",
    discussion="<para>Average the given &field; along a boundary.</para>")
   
