// -*- C++ -*-
// $RCSfile: preconditioner.C,v $
// $Revision: 1.13 $
// $Author: langer $
// $Date: 2005-05-13 18:44:06 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "engine/preconditioner.h"


ILUPreconditioner::~ILUPreconditioner()
{
  //dumpcore();
  //std::cerr << "~ILUPreconditioner()!!" << std::endl;
}

PreconditionerCore * ILUPreconditioner::build_preconditioner(const CSRmatrix &A) {
  return new ILU_Preconditioner(A);
}

PreconditionerCore * ILUPreconditioner::create_preconditioner
(const CSRmatrix &A, const std::vector<int> &rg, const std::vector<int> &cg) {
  return build_preconditioner(A);
}

ICPreconditioner::~ICPreconditioner()
{
  //dumpcore();
  //std::cerr << "~ICPreconditioner()!!" << std::endl;
}


PreconditionerCore * ICPreconditioner::build_preconditioner(const CSRmatrix &A) {
  return new IC_Preconditioner(A);
}

PreconditionerCore * ICPreconditioner::create_preconditioner
(const CSRmatrix &A, const std::vector<int> &rg, const std::vector<int> &cg) {
  return build_preconditioner(A);
}


UnPreconditioner::~UnPreconditioner()
{
  //std::cerr << "~UnPreconditioner()!!" << std::endl;
}

PreconditionerCore * UnPreconditioner::build_preconditioner(const CSRmatrix &A) {
  return new Un_conditioner(A);
}

PreconditionerCore * UnPreconditioner::create_preconditioner
(const CSRmatrix &A, const std::vector<int> &rg, const std::vector<int> &cg) {
  return build_preconditioner(A);
}


JacobiPreconditioner::~JacobiPreconditioner() {}

PreconditionerCore * JacobiPreconditioner::build_preconditioner
(const CSRmatrix &A) {
  return new Jacobi_Preconditioner(A);
}

PreconditionerCore * JacobiPreconditioner::create_preconditioner
(const CSRmatrix &A, const std::vector<int> &rg, const std::vector<int> &cg) {
  return build_preconditioner(A);
}


BlockPreconditioner::~BlockPreconditioner() {}

PreconditionerCore * BlockPreconditioner::create_preconditioner
(const CSRmatrix &A, const std::vector<int> &rg, const std::vector<int> &cg) {
  return new Block_Preconditioner(A, rg, cg, inplane);
}


const std::string Preconditioner::modulename_("oof2.SWIG.engine.preconditioner");
const std::string ILUPreconditioner::classname_("ILUPreconditioner");
const std::string ICPreconditioner::classname_("ICPreconditioner");
const std::string UnPreconditioner::classname_("UnPreconditioner");
const std::string JacobiPreconditioner::classname_("JacobiPreconditioner");
const std::string BlockPreconditioner::classname_("BlockPreconditioner");
