// -*- C++ -*-
// $RCSfile: aniso.C,v $
// $Revision: 1.15 $
// $Author: langer $
// $Date: 2007-05-10 21:22:19 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "aniso.h"
#include "engine/property/orientation/orientation.h"
#include "engine/material.h"
#include "common/tostring.h"
#include "common/trace.h"

void AnisoElasticity::cross_reference(Material *mat) {
//   Trace("AnisoElasticity::cross_reference");
  orientation = dynamic_cast<OrientationPropBase*>(mat->fetchProperty("Orientation"));
}

void AnisoElasticity::precompute(FEMesh *mesh) {
  Elasticity::precompute(mesh);
  // This assumes/requires that the rotation matrix output by
  // orientation->eulerangle() multiplies a crystal-coordinate vector
  // to give the same vector in lab coordinates.
  if(orientation->constant_in_space())
    lab_cijkl = crystal_cijkl().transform(orientation->orientation());
}

const Cijkl AnisoElasticity::cijkl(const FEMesh *mesh, const Element *el,
				   const MasterPosition &mpos)
  const
{
  if(orientation->constant_in_space())
    return lab_cijkl;
  return crystal_cijkl().transform(orientation->orientation(mesh, el, mpos));
}

const Cijkl &AnisoElasticity::crystal_cijkl() const {
  return crystal_cijkl_;
}
